import json
import requests
from urllib.parse import quote_plus

class HubSpotContactsAPI():

    def __init__(self, api_key):
        self.api_key = api_key
        self.base_url = 'https://api.hubapi.com/contacts/v1/'

    def get_contact_by_email(self, email):
        """
            Returns contact data for given email
            :email: str, contact email
        """
        encoded_email = quote_plus(email)
        url = f'{self.base_url}/contact/email/{encoded_email}/profile'
        params = {
            'hapikey': self.api_key
        }
        resp = requests.get(url, params=params)
        
        if resp.status_code == 200:
            return resp.json()
        elif resp.status_code == 404:
            return None
        else:
            raise Exception(f'this response status code is not considered before - status code: {resp.status_code}')
    
    def update_contact_by_email(self, email, properties):
        """
            Updates contact for given email properties with given properties
            :email: contact email
            :properties: list, 
            return True if updated successfully
        """
        encoded_email = quote_plus(email)
        data = json.dumps({
            'properties': properties
        })
        params = {
            'hapikey': self.api_key
        }
        url = f'{self.base_url}/contact/email/{encoded_email}/profile'
        headers = {
            'Content-Type': 'application/json'
        }
        resp = requests.post(data=data, url=url, params=params, headers=headers)
        
        if resp.status_code == 204:
            return True
        elif resp.status_code == 400:
            raise Exception(f'{resp.text}')
        elif resp.status_code == 401:
            raise Exception(f'{resp.text}')
        elif resp.status_code == 404:
            raise Exception(f'{resp.text}')
        elif resp.status_code == 500:
            raise Exception(f'{resp.text}')
        else:
            raise Exception(f'this response status code is not considered before - status code: {resp.status_code}')
        
    def create_contact(self, properties=[]):
        """
            Creates contact with given properties (email must be included)
            :properties: list
            return Response.json()
        """
        email_prop_exists = False
        for prop in properties:
            if prop['property'] == 'email' and prop['value']:
                email_prop_exists = True
        if not email_prop_exists:
            raise Exception('Email must be included')
        url = f'{self.base_url}/contact'
        params = {
            'hapikey': self.api_key
        }
        data = json.dumps({
            'properties': properties
        })
        headers = {
            'Content-Type': 'application/json'
        }
        resp = requests.post(data=data, url=url, params=params, headers=headers)

        if resp.status_code == 200:
            return resp.json()
        elif resp.status_code == 400:
            raise Exception(f'{resp.text}')
        elif resp.status_code == 409:
            raise Exception(f'{resp.text}')
        else:
            raise Exception(f'this response status code is not considered before - status code: {resp.status_code}')

    def delete_contact_by_email(self, email: str):
        """
            Deletes contact for given email
            :email: contact email
            return response.json()
        """
        contact = self.get_contact_by_email(email)
        
        if contact:
            contact_id = contact['vid']
            url = f'{self.base_url}/contact/vid/{contact_id}'
            params = {
                'hapikey': self.api_key
            }
            resp = requests.delete(url, params=params)
            
            if resp.status_code == 200:
                return resp.json()
            elif resp.status_code == 401:
                raise Exception(f'{resp.text}')
            elif resp.status_code == 404:
                raise Exception(f'{resp.text}')
            elif resp.status_code == 500:
                raise Exception(f'{resp.text}')
            else:
                raise Exception(f'this response status code is not considered before - status code: {resp.status_code}')



import json
import logging
import requests

from requests.auth import HTTPBasicAuth


class OpenSearchHelper:
    """ OpenSearch (ElasticSearch) Helper """
    GET = "get"
    PUT = "put"
    POST = "post"

    def __init__(self, url: str, index: str, username: str = None, password: str = None):
        self.url = url
        self.index = index
        self.auth = None
        self.username = username
        self.password = password

        if all([username, password]):
            self.__set_auth()

        self.__check_url()

    def get_url(self, path: str) -> str:
        """ This function return url with given path """
        return f"{self.url}/{self.index}/{path}"

    def __perform_request(self, method: str, url: str, payload: dict = {}, expected_status: int = 200) -> dict:
        """
        This function send request and check expected status after return respose
        """
        response = requests.request(
            method=method,
            url=url,
            auth=self.auth,
            headers={
                "Content-Type": "application/json"
            },
            data=json.dumps(payload)
        )

        if response.status_code != expected_status:
            raise Exception(f"Reponse Status Code -> {response.status_code} \n Message -> {response.text}")

        return response.json()

    def __set_auth(self):
        """
        This function set request authentication
        """
        self.auth = HTTPBasicAuth(username=self.username, password=self.password)

    def __check_url(self):
        """
        This function check url is working
        """
        try:
            response = self.__perform_request(method=OpenSearchHelper.GET, url=self.get_url(path="_search"))

            if not response:
                raise BaseException("URL Not Working")

        except Exception as exception:
            raise Exception(exception)

    def update_or_insert(self, data: dict, _id: str = None) -> dict:
        """
        This function insert data or update date (if has _id)
        """
        try:
            response = self.__perform_request(
                method=OpenSearchHelper.POST,
                url=self.get_url(path="_doc" if not _id else f"_doc/{_id}"),
                payload=data,
                expected_status=200 if _id else 201
            )

            return response
        except Exception as exception:
            raise Exception(exception)

    def search(self, size: int = 10, sort: str = "_id:desc", **kwargs):
        """
        This function returns search response
        """
        try:
            params = [f"{key}:{value}" for key, value in kwargs.items()]
            path = f"_search?q={','.join(params)}&size={size}&sort={sort}"
            url = self.get_url(path=path)

            response = self.__perform_request(
                method=OpenSearchHelper.GET,
                url=url,
            )

            return response
        except Exception as exception:
            raise Exception(exception)

    def get(self, payload: dict = {}):
        """
        This function is return data for payload
        """
        try:
            return requests.get(url=self.get_url(path="_search"), json=payload, auth=self.auth)
        except Exception as e:
            raise Exception(e)
